package de.evoal.core.ea.main.statistics;

import de.evoal.core.api.optimisation.OptimisationValue;
import de.evoal.core.api.properties.Properties;
import de.evoal.core.api.statistics.Candidate;
import de.evoal.core.ea.main.codec.DynamicCodec;
import io.jenetics.Phenotype;

import javax.enterprise.context.Dependent;
import javax.inject.Inject;

/**
 * An adapter, c.f. GOF: adapter pattern, for adapting Jenetic's {@link Phenotype} to
 *   EvoAl's {@link Candidate}.
 */
@Dependent
public class CandidateAdapter implements Candidate {

    /**
     * Generation for which the adapter was created for (necessary for calculating
     *   the candidate's age).
     */
    private int generation;

    /**
     * The candidate's phenotype.
     */
    private Phenotype<?, OptimisationValue> phenotype;

    @Inject
    private DynamicCodec codec;

    public CandidateAdapter init(final Phenotype<?, OptimisationValue> phenotype, final int generation) {
        this.phenotype = phenotype;
        this.generation = generation;

        return this;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Properties searchSpaceRepresentation() {
        return (Properties) codec.decoder().apply(phenotype.genotype());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int age() {
        return (int) phenotype.age(generation);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OptimisationValue value() {
        return phenotype.fitness();
    }
}
